using System;
using System.Collections.Generic;
using System.Text;
using PdfSharp.Pdf;
using PdfSharp.Pdf.Actions;

namespace PdfSharp.Pdf.Actions
{
    /// <summary>
    /// Events that happen based on user interaction
    /// </summary>
    public sealed class PdfAnnotationTriggerEvent : PdfDictionary
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="PdfAnnotationTriggerEvent"/> class.
        /// </summary>
        public PdfAnnotationTriggerEvent(PdfDocument document) : base(document)
        {
           
        }

        /// <summary>
        /// The action the occurs when your mouse goes over the annotation
        /// </summary>
        public PdfAction Enter
        {
            get
            {
                return (PdfAction)Elements.GetValue(Keys.Enter, VCF.Create);
            }
        }

        /// <summary>
        /// The action that occurs when your mouse leaves the annotation
        /// </summary>
        public PdfAction Exit
        {
            get
            {
                return (PdfAction)Elements.GetValue(Keys.Exit, VCF.Create);
            }
        }

        /// <summary>
        /// The action to perform upon pressing the mouse down
        /// </summary>
        public PdfAction MouseDown
        {
            get
            {
                return (PdfAction)Elements.GetValue(Keys.Down, VCF.Create);
            }
        }

        /// <summary>
        /// The action to perform upon releasing the mouse button
        /// </summary>
        public PdfAction MouseUp
        {
            get
            {
                return (PdfAction)Elements.GetValue(Keys.Up, VCF.Create);
            }
        }

        /// <summary>
        /// Gets the KeysMeta of this dictionary type.
        /// </summary>
        internal override DictionaryMeta Meta
        {
            get { return Keys.Meta; }
        }

        /// <summary>
        /// Predefined keys of this dictionary.
        /// </summary>
        internal class Keys : KeysBase
        {
            public static DictionaryMeta Meta
            {
                get
                {
                    if (Keys.meta == null)
                        Keys.meta = CreateMeta(typeof(Keys));
                    return Keys.meta;
                }
            }
            static DictionaryMeta meta;

            /// <summary>
            /// (Optional) An action to be performed when the cursor enters the annotations active area.
            /// </summary>
            [KeyInfo("1.2", KeyType.Dictionary | KeyType.Optional, typeof(PdfAction))]
            public const string Enter = "/E";

            /// <summary>
            /// (Optional) An action to be performed when the cursor leaves the annotations active area.
            /// </summary>
            [KeyInfo("1.2", KeyType.Dictionary | KeyType.Optional, typeof(PdfAction))]
            public const string Exit = "/X";

            /// <summary>
            /// (Optional) An action to be performed when the mouse button is pressed down inside 
            /// the annotations active area.
            /// </summary>
            [KeyInfo("1.2", KeyType.Dictionary | KeyType.Optional, typeof(PdfAction))]
            public const string Down = "/D";

            /// <summary>
            /// (Optional) An action to be performed when the mouse button is pressed up inside 
            /// the annotations active area.
            /// </summary>
            [KeyInfo("1.2", KeyType.Dictionary | KeyType.Optional, typeof(PdfAction))]
            public const string Up = "/U";

        }
    }
}
