#region PDFsharp - A .NET library for processing PDF
//
// Authors:
//   Stefan Lange (mailto:Stefan.Lange@pdfsharp.com)
//
// Copyright (c) 2005-2008 empira Software GmbH, Cologne (Germany)
//
// http://www.pdfsharp.com
// http://sourceforge.net/projects/pdfsharp
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
#endregion

using System;
using System.Globalization;
using System.Diagnostics;
using System.Collections;
using PdfSharp.Internal;
using PdfSharp.Drawing;
using PdfSharp.Pdf;
using PdfSharp.Pdf.Advanced;
using PdfSharp.Pdf.IO;
using PdfSharp.Pdf.Internal;
using System.Text;
using PdfSharp.Pdf.AcroForms;

namespace PdfSharp.Pdf.Annotations
{
    /// <summary>
    /// Represents a text annotation.
    /// </summary>
    public sealed class PdfFreeTextAnnotation : PdfAnnotation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="PdfFreeTextAnnotation"/> class.
        /// </summary>
        public PdfFreeTextAnnotation(PdfDocument document) : base(document)
        {
            Initialize();
        }

        void Initialize()
        {
            Elements.SetName(Keys.Subtype, "/FreeText");
            SetAppearance();
        }

        private XColor _textColor = XColors.Black;
        /// <summary>
        /// Set the color of the annotation text
        /// </summary>
        public XColor TextColor
        {
            get { return _textColor; }
            set { _textColor = value; }
        }

        private XColor _borderColor = XColors.Black;
        /// <summary>
        /// Set the color of the annotation border
        /// </summary>
        public XColor BorderColor
        {
            get { return _borderColor; }
            set { _borderColor = value; }
        }

        private void SetAppearance()
        {
            StringBuilder sb = new StringBuilder();

            sb.Append((_textColor.R / 255.0) + " " + (_textColor.G / 255.0) + " " + (_textColor.B / 255.0) + " rg ");
            sb.Append((_borderColor.R / 255.0) + " " + (_borderColor.G / 255.0) + " " + (_borderColor.B / 255.0) + " RG ");
            
            Elements.SetString("/DA", sb.ToString());
        }

        /// <summary>
        /// STFU
        /// </summary>
        public PdfAcroFieldFlags AcroFormFlags
        {
            get { return (PdfAcroFieldFlags)Elements.GetInteger("/Ff"); }
            set
            {
                Elements.SetInteger("/Ff", (int)value);
                Elements.SetDateTime(Keys.M, DateTime.Now);
            }
        }


        /// <summary>
        /// A code specifying the form of quadding (justification) to be 
        /// used in displaying the annotations text: 
        /// 0 Left-justified 
        /// 1 Centered 
        /// 2 Right-justified 
        /// Default value: 0 (left-justified).
        /// </summary>
        public int TextAlign
        {
            get
            {
                return Elements.GetInteger("/Q");
            }
            set
            {
                Elements.SetInteger("/Q", value);
            }
        }

        /// <summary>
        /// Predefined keys of this dictionary.
        /// </summary>
        internal new class Keys : PdfAnnotation.Keys
        {
            public static DictionaryMeta Meta
            {
                get
                {
                    if (Keys.meta == null)
                        Keys.meta = CreateMeta(typeof(Keys));
                    return Keys.meta;
                }
            }
            static DictionaryMeta meta;
        }

        /// <summary>
        /// Gets the KeysMeta of this dictionary type.
        /// </summary>
        internal override DictionaryMeta Meta
        {
            get { return Keys.Meta; }
        }
    }
}
